## 接口说明

### `QVBP2P`

QVBP2P 是sdk lib名称, 可以访问到一些常量和生成 qvbp2p实例, 如下`qvbp2p`表示sdk实例

#### `QVBP2P.create(config: Config): QVBP2P`
创建SDK的实例
- 请注意, 创建实例前请销毁上一个实例

##### 参数
```typescript
  type Config = {
    tencentCloudAppId: number; // 必填, 腾讯云AppId
    bizId: string; // 必填, 邮件给出
    xp2pAppId: string; // 必填, 邮件给出
    xp2pPackage: string; // 必填, 邮件给出
    xp2pPlayDomain: string; // 必填, 拉流域名, 邮件给出或按需填写
    xp2pAppKey: string; // 必填, 邮件给出
    authMode: 'none' | 'local' | 'remote'; // 必填, 鉴权模式. 请根据如下表格自行选择
    localSecKey?: string; // 可选, 邮件给出
    debug?: boolean; // debug开关
  }
```

> `authMode` 参数含义
> - 'none': 使用原有的拉流鉴权参数, 因为SDK运行中可能会多次重新拉流, 因此使用此选项请注意您的鉴权过期时间. 鉴权过期后会导致P2P拉流失败; 如果您的拉流域名没有防盗链,也需使用此项配置
> - 'local': 使用腾讯云的鉴权. 此选项需要您设置 `localSecKey`字段, 值为xp2pPlayDomain域名对应的鉴权key(控制台-云直播-域名管理-访问控制-Key鉴权); SDK内部会使用这个key实时生成防盗链地址
> - 'remote': 远程鉴权, 使用XP2P的鉴权方式, 需要您传入xp2pAppKey, 并且由我们来配置域名的鉴权

##### 返回值
```
QVBP2P // sdk实例
```

### 常量

#### `QVBP2P.version: string`
查看当前sdk版本

#### `QVBP2P.ComEvents`
SDK会抛出的事件,客户端需监听处理

##### 定义
```typescript
const ComEvents = {
  STATE_CHANGE: 'tp2pStateChange', // 目前sdk仅抛出了这一个事件, 客户需要监听这个事件,并处理对应的消息,消息码见 QVBP2P.ComCode
}
```

#### `QVBP2P.ComCodes`
QVBP2P.ComEvents.STATE_CHANGE 事件对对应多种消息, 通过此处的消息码区分处理
##### 定义
```typescript
const ComCode = {
  ROLLBACK: -2, // sdk回退
  RECEIVE_BUFFER: 1, // 接收flv数据chunk
  END_OF_STREAM: 3, // 流结束
  STREAM_NOT_FOUND: 4, // 启动时拉流404
};
```

- 回退说明: 当sdk拉流失败,播放长时间卡顿等异常情况时候, 会抛出此事件, 客户播放器此时需要使用原有方式重新尝试拉流,不使用P2P
- QVBP2P.ComCodes`和`QVBP2P.ComEvents`用法见示例

### 方法
#### `QVBP2P.isSupported(): boolean`
静态方法,判断当前浏览器是否支持sdk

##### 返回值

`boolean`  true表示支持sdk

##### 例子

```typescript
if (QVBP2P.isSupported()) {
  // 支持sdk,可以使用
}
```


#### `listen()`
向sdk注册监听的事件

##### 语法
```typescript
qvbp2p.listen(event, callback)
```

##### 参数
```typescript
type event = QVBP2P.ComEvents.STATE_CHANGE;
type callback = (event: QVBP2P.ComEvents.STATE_CHANGE, data: CallbackData) => void;

type ReceiveBufferCallbackData = { 
    code: ComCodes.RECEIVE_BUFFER, // 接收数据
    payload: ArrayBuffer; // flv chunk数据
}

type RollbackCallbackData = {
    code: ComCodes.ROLLBACK
    rollbackReason: string;
};

type CallbackData = ReceiveBufferCallbackData | RollbackCallbackData;
  
```

##### 返回值
```typescript
void
```

##### 例子
```
见demo
```

#### `setMediaElement(videoElement: HTMLVideoElement)`
给sdk传入当前的videoElement, sdk使用videoElement进行事件监控,buffer查询.请在`loadSource()`调用前前设置

##### 参数
```typescript
type videoElement: HTMLVideoElement;
```

##### 返回值
```typescript
void
```

##### 例子

```typescript
const videoEl = document.getElementById('your-video-id')
qvbp2p.setMediaElement(videoEl);
```

#### `loadSource(loadConfig: LoadConfig)`
启动sdk拉流功能, 此时sdk会向cdn和p2p节点进行请求数据,并通过事件返回
> __此方法不能重复调用, 只能调用一次__

##### 参数
```typescript
type LoadConfig = {
  src: string; // 原始flv流的url. sdk会进行解析, 然后拼出sdk内部使用的url进行拉流
  xresid?: string; // 可选, 如果不同拉流域名会使用相同的流id, 则可以自行设置不同xresid, 避免P2P串流
}
```

##### 返回值
```typescript
void
```

##### 例子

```typescript
qvbp2p.loadSource({
  src: 'https://xxxx.xxx.xxx/live/teststrea.flv?xxxx'
})
```

#### `destroy()`
中断拉流, 销毁sdk(不能重复调用).销毁后的sdk也不能继续使用.需要重新创建sdk实例

##### 语法
```typescript
qvbp2p.destroy();
```
