
[TOC]

# 腾讯云X-P2P H5直播SDK对接指南


## 概述
腾讯云H5-P2P直播解决方案，可帮助用户直接使用经过大规模验证的直播流媒体分发服务。用户可通过SDK中简洁的接口快速同自有应用集成，实现H5 P2P直播功能。



## 运行条件

### 1. sdk运行需要浏览器支持以下特性:

- Media Source Extention
- WEB RTC
- websocket


| 浏览器版本要求 |桌面  |
| ------------- |:--------:|
| chrome        | 55+      |
| firefox       | 65+      |
| safari        | 11+      |
| edge          | 16+      |
| IE            | 不支持    |

> 您可[点击此处](https://xp2p-1258344699.cos.ap-nanjing.myqcloud.com/demo/h5-support/index.html)查看浏览器是否支持以上特性

### 2. 支持的流媒体格式

- http-flv

## Demo示例

- 集成 SDK 到播放器（flv.js）编写 loader 示例代码，请参见 [此处](https://xp2p-1258344699.cos.ap-nanjing.myqcloud.com/demo/h5_flv_p2p/index.html)。
- 集成 SDK 到播放器（flv.js）的 Demo，请参见 [为播放器编写loader](#2-为播放器编写loader--集成sdk到播放器)一节。


## 准备工作

在 [提交 X-P2P 开通申请](https://cloud.tencent.com/apply/p/npwwbfakdis) 后，再联系我们的研发工程师，确保 CDN 分发域名及 domain 白名单已完成配置。


## 集成工作
### 1. 首先加载sdk
```
<script src='qvbp2p.js'></script>
```

### 2. 为播放器编写loader / 集成sdk到播放器

通常H5 FLV player有不同的loader用于下载, 本例子是集成到flv.js, 代码如下
```javascript
// 本示例需要提前引入flv.js和xp2p sdk

/**
   * XP2P loader, 对接flv.js
   */
class QVBP2PLoader extends flvjs.BaseLoader {
  /**
     * 确定当前环境是否支持sdk
     * @returns {boolean}
     */
  static isSupported() {
    return window.QVBP2P && window.QVBP2P.isSupported();
  }

  constructor(seekHander, config) {
    super();
    this._qvbp2p = null;
    // flv.js成员,非sdk必须
    this._receivedLength = 0;
    this._config = config;
  }

  /**
     * @public
     */
  destroy() {
    this._destroyQVBP2P();
    super.destroy();
  }

  /**
     * 通过p2p sdk播放一个url
     * @public
     * @param {object} dataSource
     * @param {string} dataSource.url
     */
  open(dataSource) {
    // 初始化sdk实例
    this._createQVBP2P();

    // 监听sdk事件
    this._qvbp2p.listen(
      window.QVBP2P.ComEvents.STATE_CHANGE,
      this._onQVBP2PStateChange.bind(this),
    );

    // 绑定video元素到sdk, 需要替换为客户实际的videoEl
    const videoEl = document.getElementById(this._config.videoId);
    this._qvbp2p.setMediaElement(videoEl);

    // 开始拉流, 注意对于一个sdk实例,loadSource不能重复调用
    const config = {
      src: dataSource.url,
    };
    this._qvbp2p.loadSource(config);
  }

  /**
     * @public
     */
  abort() {
    this._destroyQVBP2P();
  }

  /**
     * 接收sdk抛出的事件,针对不同事件类型有不同的处理
     *
     * 详细事件名称见 `QVBP2P.ComCodes`
     * @param {string} event 事件名称
     * @param {*} data
     */
  _onQVBP2PStateChange(event, data) {
    const { ComCodes } = window.QVBP2P;
    const { code } = data;
    switch (code) {
      case ComCodes.RECEIVE_BUFFER:
        this._receiveBuffer_need_to_implement(data.payload);
        break;
      case ComCodes.ROLLBACK:
        this._rollback_need_to_implement();
        break;
      default:
        break;
    }
  }

  /**
     * 接收sdk下载回来的flv chunk数据, 送给播放器
     *
     * @todo 此处需要客户根据播放器情况自行实现
     *
     * @param {ArrayBuffer} chunk flv chunk
     */
  _receiveBuffer_need_to_implement(chunk) {
    const byteStart = this._receivedLength;
    this._receivedLength += chunk.byteLength;
    if (this._onDataArrival) {
      this._onDataArrival(chunk, byteStart, this._receivedLength);
    }
  }

  /**
     * 当sdk发生错误或不能继续播放时, 会抛出此事件, 需要客户使用原来的flv流程重新播放
     *
     * @todo 此处需要客户根据播放器情况自行实现
     */
  _rollback_need_to_implement() {
    window.player.loadWithoutQVBP2P();
  }

  /**
     * 创建sdk实例
     */
  _createQVBP2P() {
    if (this._qvbp2p) {
      this._destroyQVBP2P();
    }
    this._qvbp2p = window.QVBP2P.create(this._config.xp2pConfig);
    window.qvbp2p = this._qvbp2p;
  }

  /**
     * 销毁sdk实例
     */
  _destroyQVBP2P() {
    if (this._qvbp2p) {
      this._qvbp2p.destroy();
      this._qvbp2p = null;
      window.qvbp2p = null;
    }
  }
}

/**
   * 封装了flv.js的调用, 根据配置选择是否开启XP2P
   */
class Player {
  constructor() {
    this.flvjsPlayer = null;
  }

  config({ useP2P, url, xp2pConfig }) {
    this.flvjsConfig = {
      mediaDataSource: {
        type: 'flv',
        url,
        isLive: true,
      },
      optionalConfig: {
        enableWorker: false,
        videoId: 'videoId', // 必填, <video>标签的id
        xp2pConfig,
      },
    };
    this.flvjsConfig.optionalConfig.customLoader = useP2P
      ? QVBP2PLoader
      : undefined;
  }

  loadWithoutQVBP2P() {
    this.dispose();
    this.flvjsConfig.optionalConfig.customLoader = undefined;
    this.play();
  }

  play() {
    if (this.flvjsPlayer) {
      this.dispose();
    }
    this.flvjsPlayer = flvjs.createPlayer(
      this.flvjsConfig.mediaDataSource,
      this.flvjsConfig.optionalConfig,
    );
    const element = document.getElementById(this.flvjsConfig.optionalConfig.videoId);
    this.flvjsPlayer.attachMediaElement(element);
    this.flvjsPlayer.load();
  }

  dispose() {
    this.flvjsPlayer.pause();
    this.flvjsPlayer.unload();
    this.flvjsPlayer.detachMediaElement();
    this.flvjsPlayer.destroy();
    this.flvjsPlayer = null;
  }
}

// 创建
const player = new Player();
// 配置
player.config({
  useP2P: true,
  url: '需要自行替换为flv url',
  xp2pConfig: {
    // 调试 start
    debug: false,
    // 调试 end
    tencentCloudAppId: 按邮件填写,
    bizId: '按邮件填写',
    xp2pAppId: '按邮件填写',
    xp2pPackage: '按邮件填写',
    xp2pPlayDomain: '按邮件填写或参考文档',
    xp2pAppKey: '按邮件填写或参考文档',
    authMode: '按邮件填写或参考文档,
    localSecKey: '按邮件填写或参考文档',
  },
});
// 启动
window.player.play();

```


## 开发注意事项

- 需重点关注示例中`todo`,需要您来实现
- 如果sdk触发回退的话, 请自行控制回退后的播放不再使用p2p sdk
- qvbp2p实例一次性使用, `qvbp2p.loadSource()` 和 `qvbp2p.destroy()` 不能重复调用, 如需要再次播放, 请重新创建qvbp2p实例
- 每次使用qvbp2p实例播放前, 请确保上一个`qvbp2p实例`已经调用`qvbp2p.destroy()` (如下场景需要: 切换线路,切换清晰度,回退). 


## Q&A

对SDK集成及上线使用中发现的问题和意见建议，您可通过联系客服的方式进行反馈，我们的工程师会同您联系。 

---
version: 2.1

date: 2022/04/26

---
version: 2.0

date: 2019/10/10

--- 
version: v1.4

date: 2018/11/16

---
version: v1.3

date: 2018/4/20

---

version: v1.2

date: 2018/1/15

---
version: v1.1

date: 2017/10/09


